@@ -1,6 +1,13 @@
 from django.db.backends.signals import connection_created
 
-__version__ = '2.6.0'
+__version__ = '2.12.0'
+
+
+import random
+
+
+def get_random_sequence(n):
+    return "".join(["agct"[random.randint(0, 3)] for i in range(n)])
 
 
 def import_gff(name, fn):
